@@ -28,7 +28,7 @@ from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-__VERSION__ = '0.5.2'
+__VERSION__ = '0.5.3'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -304,7 +304,7 @@ class LoreMailbox:
                         self.trailer_map[attid].update(trailers)
                     pmsg.followup_trailers.update(trailers)
                     break
-                if pmsg.has_diffstat and not pmsg.reply:
+                if not pmsg.reply:
                     # Could be a cover letter
                     pmsg.followup_trailers.update(trailers)
                     break
@@ -499,10 +499,14 @@ class LoreSeries:
         at = 1
         atterrors = list()
         for lmsg in self.patches[1:]:
-            if cherrypick is not None and at not in cherrypick:
-                at += 1
-                logger.debug('  skipped: [%s/%s] (not in cherrypick)', at, self.expected)
-                continue
+            if cherrypick is not None:
+                if at not in cherrypick:
+                    at += 1
+                    logger.debug('  skipped: [%s/%s] (not in cherrypick)', at, self.expected)
+                    continue
+                if lmsg is None:
+                    logger.critical('CRITICAL: [%s/%s] is missing, cannot cherrypick', at, self.expected)
+                    raise KeyError('Cherrypick not in series')
             if lmsg is not None:
                 if self.has_cover and covertrailers and self.patches[0].followup_trailers:
                     lmsg.followup_trailers.update(self.patches[0].followup_trailers)
@@ -692,7 +696,7 @@ class LoreSeries:
                     return None, None
                 logger.debug('Looking at %s', lmsg.full_subject)
                 lmsg.load_hashes()
-                if not len(lmsg.blob_indexes):
+                if not lmsg.blob_indexes:
                     logger.critical('ERROR: some patches do not have indexes')
                     logger.critical('       unable to create a fake-am range')
                     return None, None
@@ -887,7 +891,7 @@ class LoreMessage:
                     self.trailers.add(('Fixes', tvalue))
 
             # Do we have something that looks like a person-trailer?
-            matches = re.findall(r'^\s*([\w-]+):[ \t]+(.*<\S+>)\s*$', self.body, re.MULTILINE)
+            matches = re.findall(r'^\s*([\w-]{2,}):[ \t]+(.*<\S+>)\s*$', self.body, re.MULTILINE)
             # These are commonly part of patch/commit metadata
             badtrailers = ('from', 'author', 'cc')
             if matches:
@@ -1389,7 +1393,7 @@ class LoreSubject:
             subject = re.sub(r'^\w+:\s*\[', '[', subject)
 
         # Fix [PATCHv3] to be properly [PATCH v3]
-        subject = re.sub(r'^\[\s*(patch)(v\d+).*', '[$1 $2$3', subject, flags=re.I)
+        subject = re.sub(r'^\[\s*(patch)(v\d+)(.*)', '[\\1 \\2\\3', subject, flags=re.I)
 
         # Find all [foo] in the title
         while subject.find('[') == 0:
@@ -1918,7 +1922,7 @@ def get_msgid(cmdargs):
     matches = re.search(r'^https?://[^/]+/([^/]+)/([^/]+@[^/]+)', msgid, re.IGNORECASE)
     if matches:
         chunks = matches.groups()
-        msgid = chunks[1]
+        msgid = urllib.parse.unquote(chunks[1])
         # Infer the project name from the URL, if possible
         if chunks[0] != 'r':
             cmdargs.useproject = chunks[0]
