@@ -86,12 +86,12 @@ def mbox_to_am(mboxfile, cmdargs):
             at = 0
             for lmsg in lser.patches[1:]:
                 at += 1
-                if lmsg.msgid == msgid:
+                if lmsg and lmsg.msgid == msgid:
                     cherrypick = [at]
-                    cmdargs.cherrypick = '5'
+                    cmdargs.cherrypick = f'<{msgid}>'
                     break
             if not len(cherrypick):
-                logger.critical('Specified msgid is not present in the series, cannot cherry-pick')
+                logger.critical('Specified msgid is not present in the series, cannot cherrypick')
                 sys.exit(1)
         elif cmdargs.cherrypick.find('*') >= 0:
             # Globbing on subject
@@ -110,10 +110,14 @@ def mbox_to_am(mboxfile, cmdargs):
 
     logger.critical('Writing %s', am_filename)
     mbx = mailbox.mbox(am_filename)
-    am_mbx = lser.save_am_mbox(mbx, noaddtrailers=cmdargs.noaddtrailers,
-                               covertrailers=covertrailers, trailer_order=config['trailer-order'],
-                               addmysob=cmdargs.addmysob, addlink=cmdargs.addlink,
-                               linkmask=config['linkmask'], cherrypick=cherrypick)
+    try:
+        am_mbx = lser.save_am_mbox(mbx, noaddtrailers=cmdargs.noaddtrailers,
+                                   covertrailers=covertrailers, trailer_order=config['trailer-order'],
+                                   addmysob=cmdargs.addmysob, addlink=cmdargs.addlink,
+                                   linkmask=config['linkmask'], cherrypick=cherrypick)
+    except KeyError:
+        sys.exit(1)
+
     logger.info('---')
 
     if cherrypick is None:
@@ -153,7 +157,7 @@ def mbox_to_am(mboxfile, cmdargs):
                 logger.info('Prepared a fake commit range for 3-way merge (%.12s..%.12s)', rstart, rend)
 
     logger.critical('---')
-    if not lser.complete:
+    if not lser.complete and not cmdargs.cherrypick:
         logger.critical('WARNING: Thread incomplete!')
 
     if lser.has_cover and not cmdargs.nocover:
@@ -234,18 +238,14 @@ def mbox_to_am(mboxfile, cmdargs):
     am_mbx.close()
     if cmdargs.outdir == '-':
         logger.info('---')
-        with open(am_filename, 'r') as fh:
-            shutil.copyfileobj(fh, sys.stdout)
+        with open(am_filename, 'rb') as fh:
+            shutil.copyfileobj(fh, sys.stdout.buffer)
         os.unlink(am_filename)
 
     thanks_record_am(lser, cherrypick=cherrypick)
 
 
 def thanks_record_am(lser, cherrypick=None):
-    if not lser.complete:
-        logger.debug('Incomplete series, not tracking for thanks')
-        return
-
     # Are we tracking this already?
     datadir = b4.get_data_dir()
     slug = lser.get_slug(extended=True)
@@ -253,24 +253,39 @@ def thanks_record_am(lser, cherrypick=None):
 
     patches = list()
     at = 0
-    for pmsg in lser.patches[1:]:
-        at += 1
+    padlen = len(str(lser.expected))
+    lmsg = None
+
+    for pmsg in lser.patches:
         if pmsg is None:
+            at += 1
+            continue
+
+        if lmsg is None:
+            lmsg = pmsg
+
+        if not pmsg.has_diff:
+            # Don't care about the cover letter
+            at += 1
             continue
 
         if cherrypick is not None and at not in cherrypick:
             logger.debug('Skipped non-cherrypicked: %s', at)
+            at += 1
             continue
 
         pmsg.load_hashes()
         if pmsg.attestation is None:
             logger.debug('Unable to get hashes for all patches, not tracking for thanks')
             return
-        patches.append((pmsg.subject, pmsg.pwhash, pmsg.msgid))
 
-    lmsg = lser.patches[0]
+        prefix = '%s/%s' % (str(pmsg.counter).zfill(padlen), pmsg.expected)
+        patches.append((pmsg.subject, pmsg.pwhash, pmsg.msgid, prefix))
+        at += 1
+
     if lmsg is None:
-        lmsg = lser.patches[1]
+        logger.debug('All patches missing, not tracking for thanks')
+        return
 
     allto = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('to', [])])
     allcc = email.utils.getaddresses([str(x) for x in lmsg.msg.get_all('cc', [])])
@@ -285,6 +300,7 @@ def thanks_record_am(lser, cherrypick=None):
         'references': b4.LoreMessage.clean_header(lmsg.msg['References']),
         'sentdate': b4.LoreMessage.clean_header(lmsg.msg['Date']),
         'quote': b4.make_quote(lmsg.body, maxlines=5),
+        'cherrypick': cherrypick is not None,
         'patches': patches,
     }
     fullpath = os.path.join(datadir, filename)
@@ -511,8 +527,8 @@ def main(cmdargs):
         mbx.close()
         if cmdargs.outdir == '-':
             logger.info('---')
-            with open(threadmbox, 'r') as fh:
-                shutil.copyfileobj(fh, sys.stdout)
+            with open(threadmbox, 'rb') as fh:
+                shutil.copyfileobj(fh, sys.stdout.buffer)
             os.unlink(threadmbox)
             return
 
